local bump = require "bump/bump"
local anim8 = require "anim8/anim8"
local camera = require "hump/camera"
love.graphics.setDefaultFilter("nearest", "nearest")
local image = love.graphics.newImage("Sprites/8bitSheet.png")

local player = {
    x = 50,
    y = 50 ,
    w = 8,
    h = 8,
    img = nil,
    xVel = 0,
    yVel = 0,
    grav = -500,
    acc = 1000,
    maxSpeed = 500,
    factor = 1,
    isGrounded = false
}

Platform = {
    x = 100,
    y = 150,
    w = 300,
    h = 50
}

Platform2 = {
    x = 50,
    y = 500,
    w = 700,
    h = 80

}

function love.load()
    cam = camera()
    world = bump.newWorld()
    Player = player
    Box_1 = Platform
    Box_2 = Platform2

    world:add(Player, Player.x, Player.y, Player.w, Player.h)
    world:add(Box_1, Box_1.x, Box_1.y, Box_1.w, Box_1.h)
    world:add(Box_2, Box_2.x, Box_2.y, Box_2.w, Box_2.h)

    --anim8
    Player.img = image
    --animation = anim8.newAnimation(g('1-25', 1), 0.1)
    animation()
end

function animation()
    local g = anim8.newGrid(8, 8, Player.img:getWidth(), Player.img:getHeight())
    idle = anim8.newAnimation(g("1-13", 1), 0.1)
    run = anim8.newAnimation(g("14-15", 1), 0.1)
    air = anim8.newAnimation(g(24, 1), 0.1)
    fall = anim8.newAnimation(g(25, 1), 0.1)
    currAnim = idle
end

function collide(player, dt)
    local finalX = player.x + player.xVel * dt
    local finalY = player.y + player.yVel * dt

    local actualX, actualY, cols, len = world:move(player, finalX, finalY)

    -- collision info
    player.isGrounded = false
    for i = 1, len do
        local col = cols[i]

        if col.normal.y == -1 or col.normal.y == 1 then
            player.yVel = 0
        end

        if col.normal.y == -1 then
            player.isGrounded = true
        end
    end

    player.x = actualX
    player.y = actualY
end

function appGravity(dt)
    if Player.yVel <= Player.maxSpeed then
        Player.yVel = Player.yVel - Player.grav * dt
    end
end

function playerMove(dt)
    if love.keyboard.isScancodeDown("d") then
        currAnim = run
        if Player.xVel <= Player.maxSpeed then
            Player.xVel = Player.xVel + Player.acc * dt
        end
    elseif love.keyboard.isScancodeDown("a") then
        currAnim = run
        if Player.xVel >= -Player.maxSpeed then
            Player.xVel = Player.xVel - Player.acc * dt
        end
    else
        currAnim = idle
        --Player.xVel = 0
        
        local friction = 1000
        
        --while Player.xVel ~= 0 do
        if Player.xVel > 0 then
            Player.xVel = Player.xVel - friction * dt
            if Player.xVel < 0 then Player.xVel = 0 end
        end

        if Player.xVel < 0 then
            Player.xVel = Player.xVel + friction * dt
            if Player.xVel > 0 then Player.xVel = 0 end
        end

        --[[
        if Player.xVel == 0 then
            currAnim = idle
        end
        --]]
        --end
    end
end

function animChecker()
    if not Player.isGrounded then
        if Player.yVel ~= 0 then
            if Player.yVel < 0 then
                currAnim = air
            elseif Player.yVel > 0 then
                currAnim = fall
            end
        end
    end
end

function love.update(dt)
    currAnim:update(dt)
    playerMove(dt)
    collide(Player, dt)
    appGravity(dt)
    animChecker()
    cam:lookAt(Player.x, Player.y)
    print(Player.xVel)
end

function love.draw()
    --love.graphics.push()
    --love.graphics.scale(2,2)
    cam:attach()
        currAnim:draw(Player.img, Player.x, Player.y, nil, Player.factor, 1, 4)
        love.graphics.rectangle("line", Box_1.x, Box_1.y, Box_1.w, Box_1.h)
        love.graphics.rectangle("line", Box_2.x, Box_2.y, Box_2.w, Box_2.h)
    cam:detach()
    --love.graphics.pop()
end 

function love.keypressed(key)
    if key == "w" and Player.isGrounded then
        Player.yVel = -300
    end
    if key == "d" then
        Player.factor = 1
    elseif key == "a" then
        Player.factor = -1
    end
end

